package br.ufpe.cin.rgms.linhasdepesquisa;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.persistence.*;

import br.ufpe.cin.rgms.Facade;
import br.ufpe.cin.rgms.base.RGMSException;
import br.ufpe.cin.rgms.linhasdepesquisa.modelo.LinhaPesquisa;
import br.ufpe.cin.rgms.membro.apresentacao.AdicionarMembroServlet;
import br.ufpe.cin.rgms.membro.apresentacao.AlterarDadosMembroServlet;
import br.ufpe.cin.rgms.membro.modelo.Membro;
import br.ufpe.cin.rgms.publicacao.modelo.Publicacao;
import br.ufpe.cin.rgms.linhasdepesquisa.apresentacao.AlterarDadosLinhaPesquisaServlet;
import br.ufpe.cin.rgms.linhasdepesquisa.controle.ControleLinhaPesquisa;
import br.ufpe.cin.rgms.util.StringsUtil;

privileged public aspect LinhasPesquisaAspect {
	
	List<LinhaPesquisa> Membro.linhasDePesquisa;
	
	List<LinhaPesquisa> Publicacao.linhasPesquisa;
	
	ControleLinhaPesquisa Facade.controleLinhasPesquisa;
	
	pointcut initControleLinhasPesquisa(Facade facade) :
		execution(private Facade.new()) && this(facade);
	
	after(Facade facade) : initControleLinhasPesquisa(facade){
		facade.controleLinhasPesquisa = new ControleLinhaPesquisa();
	}
	
	pointcut insertMemberWithResearchLines(AdicionarMembroServlet adicionarMembroServlet, Membro membro) :
		call(public void Facade.inserirMembro(Membro+)) && this(adicionarMembroServlet) && args(membro);
	
	before(AdicionarMembroServlet adicionarMembroServlet, Membro membro) : 
		insertMemberWithResearchLines(adicionarMembroServlet, membro){
		
		//Mudar quando conseguir adicionar o mtodo getLinhasPesquisa com static.
		StringsUtil stringsUtil = new StringsUtil();
		
		String ids = adicionarMembroServlet.getFormfields().get("linhaspesquisa");
		
		List<LinhaPesquisa> linhasPesquisa = new ArrayList<LinhaPesquisa>();
		linhasPesquisa.addAll(stringsUtil.getLinhasPesquisa(ids));
		
		membro.setLinhasDePesquisa(linhasPesquisa);
	}
	
	pointcut changeMemberWithResearchLines(AlterarDadosMembroServlet alterarDadosMembroServlet, Membro membro) :
		call(public void Facade.alterarMembro(Membro+)) && this(alterarDadosMembroServlet) && args(membro);
	
	before(AlterarDadosMembroServlet alterarDadosMembroServlet, Membro membro) : 
		changeMemberWithResearchLines(alterarDadosMembroServlet, membro){
		
		//Mudar quando conseguir adicionar o mtodo getLinhasPesquisa com static.
		StringsUtil stringsUtil = new StringsUtil();
		
		String ids = alterarDadosMembroServlet.getFormfields().get("linhaspesquisa");
		
		List<LinhaPesquisa> linhasPesquisa = new ArrayList<LinhaPesquisa>();
		linhasPesquisa.addAll(stringsUtil.getLinhasPesquisa(ids));
		
		membro.setLinhasDePesquisa(linhasPesquisa);
	}
	
	//Como adicionar um mtodo esttico  classe? Este mtodo deveria ser esttico...
	public List<LinhaPesquisa> StringsUtil.getLinhasPesquisa(String stringIds) {
		List<String> ids = getElementsWithSplitAndTrim(stringIds);
		List<LinhaPesquisa> retorno = new ArrayList<LinhaPesquisa>();
		LinhaPesquisa linhaPesquisa = null;

		for(String id : ids){
			if(!id.trim().equals("") && (linhaPesquisa = Facade.getInstance().getLinhaPesquisa(Integer.parseInt(id.trim()))) != null){
				retorno.add(linhaPesquisa);
			}
		}

		return retorno;
	}
	
	@ManyToMany
	@JoinTable(
			name="Membro_LinhaPesquisa",
	        joinColumns=@JoinColumn(name="membro_id"),
	        inverseJoinColumns=@JoinColumn(name="linha_pesquisa_id")
	)
	public List<LinhaPesquisa> Membro.getLinhasDePesquisa() {
		return linhasDePesquisa;
	}

	public void Membro.setLinhasDePesquisa(List<LinhaPesquisa> linhasDePesquisa) {
		this.linhasDePesquisa = linhasDePesquisa;
	}
	
	@ManyToMany
	@JoinTable(
			name="Publicacao_LinhaPesquisa",
	        joinColumns=@JoinColumn(name="publicacao_id"),
	        inverseJoinColumns=@JoinColumn(name="linha_pesquisa_id")
	)
	public List<LinhaPesquisa> Publicacao.getLinhasPesquisa() {
		return linhasPesquisa;
	}

	public void Publicacao.setLinhasPesquisa(List<LinhaPesquisa> linhasPesquisa) {
		this.linhasPesquisa = linhasPesquisa;
	}
	
	public List<LinhaPesquisa> Facade.getLinhasPesquisa(){
		return this.controleLinhasPesquisa.listar();
	}
	
	public void Facade.inserirLinhaPesquisa(LinhaPesquisa linhaPesquisa) throws RGMSException {
		this.controleLinhasPesquisa.inserir(linhaPesquisa);
	}
	
	public LinhaPesquisa Facade.getLinhaPesquisa(int idLinha){
		return this.controleLinhasPesquisa.getLinhaPesquisa(idLinha);
	}
	
	public void Facade.alterarLinhaPesquisa(LinhaPesquisa linhaParaAlterar) throws RGMSException {
		this.controleLinhasPesquisa.alterar(linhaParaAlterar);
	}
	
	public List<LinhaPesquisa> Facade.getLinhasPesquisa(HashMap<String, String> formfields) {
		return this.controleLinhasPesquisa.getLinhasPesquisa(formfields);
	}
	
	public void Facade.removerLinhaPesquisa(LinhaPesquisa linhaPesquisa){
		this.controleLinhasPesquisa.remover(linhaPesquisa);
	}
}
